#ifndef ASAPO_GENERIC_SERVICE_REQUEST_H
#define ASAPO_GENERIC_SERVICE_REQUEST_H

#include <chrono>
#include "asapo/common/networking.h"
#include "asapo/common/data_structs.h"
#include "asapo/io/io.h"
#include "asapo/common/networking.h"
#include "asapo/http_client/http_client.h"

#include <mutex>
#include <atomic>

namespace asapo
{

    namespace ServiceName {
        const std::string kBroker = "asapo-broker";
        const std::string kFileTransfer = "asapo-file-transfer";
    }   

    enum class OutputDataMode
    {
        string,
        array,
        file
    };

    struct RequestInfo
    {
        std::string host;
        std::string api;
        std::string extra_params;
        std::string body;
        std::string cookie;
        OutputDataMode output_mode = OutputDataMode::string;
        bool post = false;
    };

    struct RequestOutput
    {
        std::string string_output;
        MessageData data_output;
        uint64_t data_output_size;
        const char *to_string() const
        {
            if (!data_output)
            {
                return string_output.c_str();
            }
            else
            {
                return reinterpret_cast<char const *>(data_output.get());
            }
        }
    };

    Error ProcessRequestResponce(const RequestInfo& request, Error server_err, const RequestOutput* response,
                             const HttpCode& code);
    Error ConsumerErrorFromNoDataResponse(const std::string& response);
    Error ConsumerErrorFromPartialDataResponse(const std::string& response);
    DataSet DecodeDatasetFromResponse(std::string response, Error* err);


    class ServiceRequest
    {
    public:
        std::string UrlEscape(std::string url);
        ServiceRequest(std::string server_uri, SourceCredentials source);

        std::string BrokerRequestWithTimeout(RequestInfo request, Error *err, uint64_t timeout_ms);
        RequestInfo CreateBrokerApiRequest(std::string stream, std::string group, std::string suffix) const;
        Error ServiceRequestWithoutTimeout(const std::string &service_name, std::string *service_uri,
                                           RequestInfo request, RequestOutput *response);
        Error ServiceRequestWithTimeout(const std::string& service_name, std::string* service_uri, RequestInfo request,
                                        RequestOutput* response, uint64_t timeout_ms);

        Error ProcessRequest(RequestOutput *response, const RequestInfo &request, std::string *service_uri);

        Error GetServerVersionInfo(std::string client_type, std::string client_protocol_version,
                                   std::string* server_info, bool* supported);

        std::unique_ptr<HttpClient> httpclient__;
        void InterruptCurrentOperation();
    private:
        std::string RequestWithToken(std::string uri);
        
        Error ProcessPostRequest(const RequestInfo &request, RequestOutput *response, HttpCode *code);
        Error ProcessGetRequest(const RequestInfo &request, RequestOutput *response, HttpCode *code);

        Error DiscoverService(const std::string& service_name, std::string* uri_to_set);        
        RequestInfo GetDiscoveryRequest(const std::string& service_name) const;
        Error ProcessDiscoverServiceResult(Error err, std::string *uri_to_set);

        std::string endpoint_;
        SourceCredentials source_credentials_;
        std::string data_source_encoded_;
        std::string current_broker_uri_;
        std::atomic<bool> interrupt_flag_{false};

    };
}
#endif // ASAPO_GENERIC_SERVICE_REQUEST_H